@@ -216,6 +216,72 @@ func TestKernelArguments(t *testing.T) {
 	}
 }
 
+func TestKernelType(t *testing.T) {
+	cs := framework.NewClientSet("")
+	// Get initial MachineConfig used by the worker pool so that we can rollback to it later on
+	mcp, err := cs.MachineConfigPools().Get("worker", metav1.GetOptions{})
+	require.Nil(t, err)
+	workerOldMC := mcp.Status.Configuration.Name
+
+	kernelType := &mcfgv1.MachineConfig{
+		ObjectMeta: metav1.ObjectMeta{
+			Name:   fmt.Sprintf("kerneltype-%s", uuid.NewUUID()),
+			Labels: mcLabelForWorkers(),
+		},
+		Spec: mcfgv1.MachineConfigSpec{
+			Config:     ctrlcommon.NewIgnConfig(),
+			KernelType: "realtime",
+		},
+	}
+
+	_, err = cs.MachineConfigs().Create(kernelType)
+	require.Nil(t, err)
+	t.Logf("Created %s", kernelType.Name)
+	renderedConfig, err := waitForRenderedConfig(t, cs, "worker", kernelType.Name)
+	require.Nil(t, err)
+	if err := waitForPoolComplete(t, cs, "worker", renderedConfig); err != nil {
+		t.Fatal(err)
+	}
+	nodes, err := getNodesByRole(cs, "worker")
+	require.Nil(t, err)
+	for _, node := range nodes {
+		assert.Equal(t, node.Annotations[constants.CurrentMachineConfigAnnotationKey], renderedConfig)
+		assert.Equal(t, node.Annotations[constants.MachineConfigDaemonStateAnnotationKey], constants.MachineConfigDaemonStateDone)
+
+		kernelInfo := execCmdOnNode(t, cs, node, "uname", "-a")
+		if !strings.Contains(kernelInfo, "PREEMPT RT") {
+			t.Fatalf("Node %s doesn't have expected kernel", node.Name)
+		}
+		t.Logf("Node %s has expected kernel", node.Name)
+	}
+
+	// Delete the applied kerneltype MachineConfig to make sure rollback works fine
+	if err := cs.MachineConfigs().Delete(kernelType.Name, &metav1.DeleteOptions{}); err != nil {
+		t.Error(err)
+	}
+
+	t.Logf("Deleted MachineConfig %s", kernelType.Name)
+
+	// Wait for the mcp to rollback to previous config
+	if err := waitForPoolComplete(t, cs, "worker", workerOldMC); err != nil {
+		t.Fatal(err)
+	}
+
+	// Re-fetch the worker nodes for updated annotations
+	nodes, err = getNodesByRole(cs, "worker")
+	require.Nil(t, err)
+	for _, node := range nodes {
+		assert.Equal(t, node.Annotations[constants.CurrentMachineConfigAnnotationKey], workerOldMC)
+		assert.Equal(t, node.Annotations[constants.MachineConfigDaemonStateAnnotationKey], constants.MachineConfigDaemonStateDone)
+		kernelInfo := execCmdOnNode(t, cs, node, "uname", "-a")
+		if strings.Contains(kernelInfo, "PREEMPT RT") {
+			t.Fatalf("Node %s did not rollback successfully", node.Name)
+		}
+		t.Logf("Node %s has successfully rolled back", node.Name)
+	}
+
+}
+
 func TestPoolDegradedOnFailToRender(t *testing.T) {
 	cs := framework.NewClientSet("")
 
